#include <d3d8.h>

#include ".\GSGame.h"
#include ".\GSMenu.h"
#include ".\GSDone.h"
#include ".\LD12.h"



CGSGame::CGSGame() :
  m_pCurrentObject( NULL )
{

  m_AvailableItems.push_back( LD12::GO_TIMEBOMB );
  m_AvailableItems.push_back( LD12::GO_PLASTIC_FENCE );
  m_AvailableItems.push_back( LD12::GO_WOODEN_FENCE );

  m_CurrentItem       = 0;

  m_BuildUpMode       = true;
  m_ButtonReleased    = false;
  m_Money             = 0;
  m_MoneyAtLevelStart = 0;
  m_CurrentStartDelay = 0;
  m_Done              = false;

  PrepareLevel( 1 );

}



CGSGame::~CGSGame()
{
}



void CGSGame::PrepareLevel( int Level )
{

  m_MoneyAtLevelStart = m_Money;

  m_DraggingBomb    = false;
  m_CurLevel        = Level;
  m_pCurrentObject  = NULL;
  m_CurrentStartDelay = 0;
  m_CurrentItem       = 0;
  m_Done              = false;
  m_DamageDone        = 0;
  m_DisplayNameTime   = 0.0f;

  m_Level.Load( m_CurLevel );

  m_Money += m_Level.m_MoneyAvailable;

  if ( !m_Level.m_Name.empty() )
  {
    m_DisplayNameTime = 5.0f;
  }

  m_CurrentSetup.clear();
  m_BuiltItems.clear();

  if ( m_Level.m_CurrentHouseTiles == 0 )
  {
    // no more levels
    theApp.NextState( new CGSDone() );
  }

}



void CGSGame::Display( IDirect3DDevice8* pDevice )
{

  D3DVIEWPORT8    ViewPort;

  ViewPort.X = 0;
  ViewPort.Y = 0;
  ViewPort.Width = 640;
  ViewPort.Height = 440;
  ViewPort.MinZ = 0.0f;
  ViewPort.MaxZ = 1.0f;

  theApp.m_pd3dDevice->SetViewport( &ViewPort );

  m_Level.Render();

  // Display bomb timers
  Level::tListObjects::iterator   itO( m_Level.m_Objects.begin() );
  while ( itO != m_Level.m_Objects.end() )
  {
    GameObject& Obj( *itO );

    if ( Obj.m_Type == LD12::GO_TIMEBOMB )
    {
      char    Temp[50];
      wsprintf( Temp, "%d", (int)( Obj.m_StartDelay * 10.0f ) );
      theApp.Print( Obj.m_DispX + 5, Obj.m_DispY + 2 - (int)m_Level.m_LevelDisplayOffset, Temp );
    }

    ++itO;
  }

  theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
  theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );

  theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
  theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
  theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );

  theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
  theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
  theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );

  // display clipped stuff

  ViewPort.Height = 480;
  theApp.m_pd3dDevice->SetViewport( &ViewPort );

  char    Temp[200];
  wsprintf( Temp, "$%d", m_Money );
  theApp.PrintNice( 5, 444, Temp, 0xffc0c0c0 );

  if ( m_BuildUpMode )
  {
    theApp.RenderButton( 120, 440, 120, 40, LD12::TEX_GUI_BUTTON_1, LD12::TEX_GUI_BUTTON_2, LD12::TEX_GUI_BUTTON_3 );
    theApp.PrintNiceCentered( 120 + 60, 444, "Go!" );
  }
  else
  {
    theApp.PrintNice( 120, 444, "Action!" );
    theApp.RenderTextureSection( 120, 440, theApp.m_TexSec[LD12::TEX_GUI_BUTTON_1] );
    theApp.PrintNiceCentered( 120 + 60, 444, "Go!" );
  }

  wsprintf( Temp, "Required: %d%%", m_Level.m_PercentageNeeded );
  theApp.PrintNiceRightAligned( 640 - 10, 444, Temp, 0xffc0c0c0 );
  /*
  wsprintf( Temp, "Structure Left: %d%%", (int)( m_Level.m_CurrentHouseTiles * 100 / m_Level.m_InitialHouseTiles ) );
  theApp.PrintNice( 440, 444, Temp );
  */

  /*
  wsprintf( Temp, "Damage worth to other: %d", CalculateOtherDamage() );
  theApp.PrintNiceCentered( 320, 4, Temp );
  */

  {
    GameObject*   pTempObj = m_Level.CreateObject( m_AvailableItems[m_CurrentItem] );

    theApp.RenderTextureSection( 320, 444, theApp.m_TexSec[pTempObj->m_Texture] );

    if ( m_AvailableItems[m_CurrentItem] == LD12::GO_TIMEBOMB )
    {
      wsprintf( Temp, "%d", m_CurrentStartDelay );
      theApp.Print( 320 + 9 + 5, 444 + 12 + 2, Temp );
    }

    m_Level.RemoveObject( pTempObj );
  }

  if ( m_Done )
  {
    // render unusable buttons
    theApp.RenderTextureSection( 260, 440, theApp.m_TexSec[LD12::TEX_GUI_PLUS_1] );
    theApp.RenderTextureSection( 260, 460, theApp.m_TexSec[LD12::TEX_GUI_MINUS_1] );
    theApp.RenderTextureSection( 360, 440, theApp.m_TexSec[LD12::TEX_GUI_PLUS_1] );
    theApp.RenderTextureSection( 360, 460, theApp.m_TexSec[LD12::TEX_GUI_MINUS_1] );

    theApp.RenderTextureSection( 360, 440, theApp.m_TexSec[LD12::TEX_GUI_PLUS_1] );
  }
  else
  {
    theApp.RenderButton( 260, 440, 32, 20, LD12::TEX_GUI_PLUS_1, LD12::TEX_GUI_PLUS_2, LD12::TEX_GUI_PLUS_3 );
    theApp.RenderButton( 260, 460, 32, 20, LD12::TEX_GUI_MINUS_1, LD12::TEX_GUI_MINUS_2, LD12::TEX_GUI_MINUS_3 );

    if ( m_AvailableItems[m_CurrentItem] == LD12::GO_TIMEBOMB )
    {
      theApp.RenderButton( 360, 440, 32, 20, LD12::TEX_GUI_PLUS_1, LD12::TEX_GUI_PLUS_2, LD12::TEX_GUI_PLUS_3 );
      theApp.RenderButton( 360, 460, 32, 20, LD12::TEX_GUI_MINUS_1, LD12::TEX_GUI_MINUS_2, LD12::TEX_GUI_MINUS_3 );
    }
  }

  if ( ( m_DraggingBomb )
  &&   ( m_pCurrentObject )
  &&   ( m_BuildUpMode ) )
  {
    m_pCurrentObject->Render( m_Level );

    if ( m_pCurrentObject->m_Type == LD12::GO_TIMEBOMB )
    {
      char    Temp[50];
      wsprintf( Temp, "%d", (int)( m_pCurrentObject->m_StartDelay * 10.0f ) );
      theApp.Print( m_pCurrentObject->m_DispX + 5, m_pCurrentObject->m_DispY + 2 - (int)m_Level.m_LevelDisplayOffset, Temp );
    }
  }

  // display summary
  if ( m_Done )
  {
    int     YOffset = 60;

    theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
    theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, FALSE );
    theApp.m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
    theApp.m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_SELECTARG1 );
    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_DIFFUSE );

    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1 );
    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE );
    theApp.RenderQuad( 100, YOffset, 440, 320, 0x80000000 );

    theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
    theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );

    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );

    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
    theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );

    char    Temp[50];

    theApp.PrintNice( 100 + 20, YOffset + 20, "Building structure left:", 0xffd0d0d0 );
    wsprintf( Temp, "%d", (int)( m_Level.m_CurrentHouseTiles * 100 / m_Level.m_InitialHouseTiles ) );
    theApp.PrintNiceRightAligned( 540 - 20, YOffset + 20, Temp, 0xffc0c0c0 );

    theApp.PrintNice( 100 + 20, YOffset + 60, "Minimum demolishment required:", 0xffd0d0d0 );
    wsprintf( Temp, "%d", m_Level.m_PercentageNeeded );
    theApp.PrintNiceRightAligned( 540 - 20, YOffset + 60, Temp, 0xffc0c0c0 );

    theApp.PrintNice( 100 + 20, YOffset + 120, "Damage's worth to other buildings:", 0xffd0d0d0 );
    wsprintf( Temp, "%d", m_DamageDone );
    theApp.PrintNiceRightAligned( 540 - 20, YOffset + 120, Temp, 0xffc0c0c0 );

    theApp.PrintNice( 100 + 20, YOffset + 160, "Money left:", 0xffd0d0d0 );
    wsprintf( Temp, "%d", m_Money - m_DamageDone );
    theApp.PrintNiceRightAligned( 540 - 20, YOffset + 160, Temp, 0xffc0c0c0 );

    if ( ( (int)( m_Level.m_CurrentHouseTiles * 100 / m_Level.m_InitialHouseTiles ) <= m_Level.m_PercentageNeeded )
    &&   ( m_DamageDone <= m_Money ) )
    {
      theApp.PrintNiceCentered( 320, YOffset + 210, "YOU SUCCEEDED", 0xff40c040 );

      theApp.RenderButton( 320 - 130, YOffset + 260, 120, 40, LD12::TEX_GUI_BUTTON_1, LD12::TEX_GUI_BUTTON_2, LD12::TEX_GUI_BUTTON_3 );
      theApp.PrintNiceCentered( 320 - 70, YOffset + 265, "NEXT STAGE" );

      theApp.RenderButton( 320 + 10, YOffset + 260, 120, 40, LD12::TEX_GUI_BUTTON_1, LD12::TEX_GUI_BUTTON_2, LD12::TEX_GUI_BUTTON_3 );
      theApp.PrintNiceCentered( 320 + 70, YOffset + 265, "RETRY" );
    }
    else
    {
      theApp.PrintNiceCentered( 320, YOffset + 210, "YOU FAILED", 0xffc04040 );
      theApp.RenderButton( 260, YOffset + 260, 120, 40, LD12::TEX_GUI_BUTTON_1, LD12::TEX_GUI_BUTTON_2, LD12::TEX_GUI_BUTTON_3 );
      theApp.PrintNiceCentered( 320, YOffset + 265, "TRY AGAIN" );
    }
    //theApp.PrintNiceCenteredWavy( 320, 220, "WELL DONE" );
  }
  if ( m_DisplayNameTime > 0.0f )
  {
    theApp.PrintNiceCentered( 321, 200, m_Level.m_Name.c_str(), 0xff000000 );
    theApp.PrintNiceCentered( 319, 200, m_Level.m_Name.c_str(), 0xff000000 );
    theApp.PrintNiceCentered( 320, 199, m_Level.m_Name.c_str(), 0xff000000 );
    theApp.PrintNiceCentered( 320, 201, m_Level.m_Name.c_str(), 0xff000000 );

    theApp.PrintNiceCentered( 320, 200, m_Level.m_Name.c_str() );
  }
  if ( m_Level.m_Height > 15 )
  {
    theApp.RenderTextureSection( 640 - 32, 0, theApp.m_TexSec[LD12::TEX_GUI_ARROW_UP] );
    theApp.RenderTextureSection( 640 - 32, 440 - 32, theApp.m_TexSec[LD12::TEX_GUI_ARROW_DOWN] );
  }
  if ( ( !m_Done )
  &&   ( m_BuildUpMode ) )
  {
    if ( ( theApp.m_MouseX >= 316 )
    &&   ( theApp.m_MouseX < 316 + 36 )
    &&   ( theApp.m_MouseY >= 440 )
    &&   ( theApp.m_MouseY < 440 + 40 ) )
    {
      // info popup over item
      theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
      theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, FALSE );
      theApp.m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
      theApp.m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_SELECTARG1 );
      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_DIFFUSE );

      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1 );
      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE );
      theApp.RenderQuad( 220, 360, 200, 80, 0x80000000 );

      theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
      theApp.m_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );

      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );

      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
      theApp.m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );

      switch ( m_AvailableItems[m_CurrentItem] )
      {
        case LD12::GO_TIMEBOMB:
          theApp.Print( 225, 365, "Timed Bomb" );
          theApp.Print( 225, 380, "Circular explosion radius" );
          break;
        case LD12::GO_PLASTIC_FENCE:
          theApp.Print( 225, 365, "Plastic Sheeting" );
          theApp.Print( 225, 380, "Basic debris protection" );
          break;
        case LD12::GO_WOODEN_FENCE:
          theApp.Print( 225, 365, "Wooden Board Sheeting" );
          theApp.Print( 225, 380, "Sturdy debris protection" );
          break;
      }
      wsprintf( Temp, "Price: %d$", ItemPrice( m_AvailableItems[m_CurrentItem] ) );
      theApp.Print( 225, 420, Temp );
    }
  }

}



int CGSGame::CalculateOtherDamage()
{

  int       DamageToStructure       = 0;
  int       DamageToInnerStructure  = 0;
  int       CollapsedTiles          = m_Level.m_InitialOtherHouseTiles - m_Level.m_CurrentOtherHouseTiles;

  for ( int i = 0; i < 20; ++i )
  {
    for ( int j = 0; j < m_Level.m_Height; ++j )
    {
      LD12::TileTypes Tile = m_Level.GetField( i, j );

      if ( ( Tile == LD12::TILE_OTHER_WINDOW_1 )
      ||   ( Tile == LD12::TILE_OTHER_WINDOW_1_BROKEN_1 )
      ||   ( Tile == LD12::TILE_OTHER_WINDOW_1_BROKEN_2 )
      ||   ( Tile == LD12::TILE_OTHER_WINDOW_1_BROKEN_3 )
      ||   ( Tile == LD12::TILE_OTHER_WINDOW_1_BROKEN_4 )
      ||   ( Tile == LD12::TILE_OTHER_WINDOW_1_BROKEN_5 ) )
      {
        Level::TileInfo& Struct = m_Level.Structure( i, j );

        DamageToStructure       += 100 - Struct.Strength;
        DamageToInnerStructure  += 100 - Struct.InnerStrength;
      }
    }
  }
  return DamageToStructure * 2 + DamageToInnerStructure * 10 + CollapsedTiles * 2500;

}



void CGSGame::UpdateFrame( const float fElapsedTime )
{

  if ( m_DisplayNameTime > 0.0f )
  {
    m_DisplayNameTime -= fElapsedTime;
    if ( m_DisplayNameTime <= 0.0f )
    {
      m_DisplayNameTime = 0.0f;
    }
    return;
  }

  if ( m_Done )
  {
  }
  else if ( !m_BuildUpMode )
  {
    m_Level.Update( fElapsedTime );
    /*
    if ( (int)( m_Level.m_CurrentHouseTiles * 100 / m_Level.m_InitialHouseTiles ) <= m_Level.m_PercentageNeeded )
    {
      m_Done = true;
    }
    else*/ 
    if ( m_Level.m_Objects.empty() )
    {
      m_Done = true;
      m_DamageDone = CalculateOtherDamage();
    }
  }
  else
  {
    m_Level.PlayRequestedSounds();
  }
  if ( ( theApp.m_bKeyPressed[VK_UP] )
  ||   ( ( theApp.m_MouseX >= 640 - 32 )
  &&     ( theApp.m_MouseY < 32 ) ) )
  {
    if ( m_Level.m_LevelDisplayOffset > 0.0f )
    {
      m_Level.m_LevelDisplayOffset -= 320.0f * fElapsedTime;
      if ( m_Level.m_LevelDisplayOffset < 0.0f )
      {
        m_Level.m_LevelDisplayOffset = 0.0f;
      }
    }
  }
  else if ( ( theApp.m_bKeyPressed[VK_DOWN] )
  ||        ( ( theApp.m_MouseX >= 640 - 32 )
  &&          ( theApp.m_MouseY >= 440 - 32 )
  &&          ( theApp.m_MouseY < 440 ) ) )
  {
    if ( m_Level.m_LevelDisplayOffset < m_Level.m_Height * 32.0f - 440 )
    {
      m_Level.m_LevelDisplayOffset += 320.0f * fElapsedTime;
      if ( m_Level.m_LevelDisplayOffset > m_Level.m_Height * 32.0f - 440 )
      {
        m_Level.m_LevelDisplayOffset = m_Level.m_Height * 32.0f - 440;
      }
    }
  }

}



void CGSGame::OnChar( int iChar )
{

  if ( iChar == 27 )
  {
    // TODO - submenu!
    theApp.NextState( new CGSMenu() );
  }
  else if ( iChar == 'n' )
  {
    m_CurLevel++;
    PrepareLevel( m_CurLevel );
    m_Done        = false;
    m_BuildUpMode = true;
  }

}



void CGSGame::OnKeyDown( int iChar )
{

  if ( iChar == VK_DOWN )
  {
  }

}



void CGSGame::RecalcStructures()
{

  for ( int i = 0; i < 20; ++i )
  {
    for ( int j = 0; j < m_Level.m_Height; ++j )
    {
      Level::TileInfo&  Struct  = m_Level.Structure( i, j );
      LD12::TileTypes   Tile    = m_Level.GetField( i, j );

      switch ( Tile )
      {
        case LD12::TILE_PLASTIC_FENCE:
          Struct.Strength = 25;
          break;
        case LD12::TILE_WOODEN_FENCE:
        case LD12::TILE_WOODEN_FENCE_TOP:
          Struct.Strength = 140;
          break;
        default:
          Struct = Level::TileInfo();
          break;
      }
    }
  }

}



void CGSGame::RetryLevel()
{

  float   PreviousOffset = m_Level.m_LevelDisplayOffset;

  // Back to build up mode
  m_BuildUpMode = true;
  m_Done        = false;
  m_Level.Load( m_CurLevel );
  // reset our setup
  m_Level.m_Objects = m_CurrentSetup;
  
  std::map<std::pair<int,int>,LD12::TileTypes>::iterator    it( m_BuiltItems.begin() );
  while ( it != m_BuiltItems.end() )
  {
    m_Level.SetField( it->first.first, it->first.second, it->second );

    if ( ( it->second == LD12::TILE_WOODEN_FENCE )
    &&   ( m_Level.GetField( it->first.first, it->first.second - 1 ) != LD12::TILE_WOODEN_FENCE )
    &&   ( m_Level.GetField( it->first.first, it->first.second - 1 ) != LD12::TILE_WOODEN_FENCE_TOP ) )
    {
      m_Level.SetField( it->first.first, it->first.second, LD12::TILE_WOODEN_FENCE_TOP );
    }

    ++it;
  }

  // readjust offset
  m_Level.m_LevelDisplayOffset = PreviousOffset;

}



void CGSGame::OnMouse( int iX, int iY, int iButtons )
{

  if ( ( iButtons & 1 )
  &&   ( m_ButtonReleased ) )
  {
    m_ButtonReleased = false;

    if ( m_DisplayNameTime > 0.0f )
    {
      m_DisplayNameTime = 0.0f;
      return;
    }

    if ( m_BuildUpMode )
    {
      if ( m_AvailableItems[m_CurrentItem] == LD12::GO_TIMEBOMB )
      {
        if ( ( iX >= 360 )
        &&   ( iX < 360 + 32 )
        &&   ( iY >= 440 )
        &&   ( iY < 440 + 20 ) )
        {
          // Plus
          if ( m_CurrentStartDelay < 100 )
          {
            ++m_CurrentStartDelay;
          }
        }
        if ( ( iX >= 360 )
        &&   ( iX < 360 + 32 )
        &&   ( iY >= 460 )
        &&   ( iY < 460 + 20 ) )
        {
          // minus
          if ( m_CurrentStartDelay >0 )
          {
            --m_CurrentStartDelay;
          }
        }
      }
      if ( ( iX >= 260 )
      &&   ( iX < 260 + 32 )
      &&   ( iY >= 440 )
      &&   ( iY < 440 + 20 ) )
      {
        // next item
        m_CurrentItem = ( ( m_CurrentItem + 1 ) % m_AvailableItems.size() );
      }
      if ( ( iX >= 260 )
      &&   ( iX < 260 + 32 )
      &&   ( iY >= 460 )
      &&   ( iY < 460 + 20 ) )
      {
        // previous item
        m_CurrentItem = ( ( m_CurrentItem + m_AvailableItems.size() - 1 ) % m_AvailableItems.size() );
      }

      if ( !m_DraggingBomb )
      {
        if ( ( iX >= 316 )
        &&   ( iX < 366 )
        &&   ( iY >= 440 )
        &&   ( iY < 480 ) )
        {
          // Check money!
          if ( m_Money >= ItemPrice( m_AvailableItems[m_CurrentItem] ) )
          {
            m_Money -= ItemPrice( m_AvailableItems[m_CurrentItem] );
            m_DraggingBomb = true;
            m_pCurrentObject                = m_Level.CreateObject( m_AvailableItems[m_CurrentItem] );
            m_pCurrentObject->m_DispX       = iX;
            m_pCurrentObject->m_DispY       = iY;
            m_pCurrentObject->m_Hidden      = true;
            if ( m_AvailableItems[m_CurrentItem] == LD12::GO_TIMEBOMB )
            {
              m_pCurrentObject->m_StartDelay  = m_CurrentStartDelay * 0.1f;
            }
            m_Level.m_SoundRequests.insert( LD12::SOUND_GET_ITEM );
          }
        }
        else if ( iY < 440 )
        {
          // drag already setup bomb?
          int TX = iX / 32;
          int TY = ( iY + (int)m_Level.m_LevelDisplayOffset ) / 32;

          if ( m_Level.GetField( TX, TY ) == LD12::TILE_PLASTIC_FENCE )
          {
            // auto-pick all built fences above
            int   CY = TY - 1;
            while ( ( CY >= 0 )
            &&      ( m_Level.GetField( TX, CY ) != LD12::TILE_SKY_EMPTY ) )
            {
              if ( ( m_Level.GetField( TX, CY ) == LD12::TILE_PLASTIC_FENCE )
              ||   ( m_Level.GetField( TX, CY ) == LD12::TILE_WOODEN_FENCE )
              ||   ( m_Level.GetField( TX, CY ) == LD12::TILE_WOODEN_FENCE_TOP ) )
              {
                LD12::GameObjectType    GOType = LD12::GO_WOODEN_FENCE;
                if ( m_Level.GetField( TX, CY ) == LD12::TILE_PLASTIC_FENCE )
                {
                  GOType = LD12::GO_PLASTIC_FENCE;
                }
                m_Money += ItemPrice( GOType );

                std::map<std::pair<int,int>,LD12::TileTypes>::iterator    itBI( m_BuiltItems.find( std::make_pair( TX, CY ) ) );
                if ( itBI != m_BuiltItems.end() )
                {
                  m_BuiltItems.erase( itBI );
                }
                m_Level.SetField( TX, CY, LD12::TILE_SKY_EMPTY );
              }
              CY--;
            }
            m_Level.SetField( TX, TY, LD12::TILE_SKY_EMPTY );
            if ( m_Level.GetField( TX, TY + 1 ) == LD12::TILE_WOODEN_FENCE )
            {
              m_Level.SetField( TX, TY + 1, LD12::TILE_WOODEN_FENCE_TOP );
            }
            m_DraggingBomb = true;
            m_pCurrentObject = m_Level.CreateObject( LD12::GO_PLASTIC_FENCE, iX, iY + (int)m_Level.m_LevelDisplayOffset );
            std::map<std::pair<int,int>,LD12::TileTypes>::iterator    itBI( m_BuiltItems.find( std::make_pair( TX, TY ) ) );
            if ( itBI != m_BuiltItems.end() )
            {
              m_BuiltItems.erase( itBI );
            }
            m_Level.m_SoundRequests.insert( LD12::SOUND_GET_ITEM );
          }
          else if ( ( m_Level.GetField( TX, TY ) == LD12::TILE_WOODEN_FENCE )
          ||        ( m_Level.GetField( TX, TY ) == LD12::TILE_WOODEN_FENCE_TOP ) )
          {
            // auto-pick all built fences above
            int   CY = TY - 1;
            while ( ( CY >= 0 )
            &&      ( m_Level.GetField( TX, CY ) != LD12::TILE_SKY_EMPTY ) )
            {
              if ( ( m_Level.GetField( TX, CY ) == LD12::TILE_PLASTIC_FENCE )
              ||   ( m_Level.GetField( TX, CY ) == LD12::TILE_WOODEN_FENCE )
              ||   ( m_Level.GetField( TX, CY ) == LD12::TILE_WOODEN_FENCE_TOP ) )
              {
                LD12::GameObjectType    GOType = LD12::GO_WOODEN_FENCE;
                if ( m_Level.GetField( TX, CY ) == LD12::TILE_PLASTIC_FENCE )
                {
                  GOType = LD12::GO_PLASTIC_FENCE;
                }
                m_Money += ItemPrice( GOType );

                std::map<std::pair<int,int>,LD12::TileTypes>::iterator    itBI( m_BuiltItems.find( std::make_pair( TX, CY ) ) );
                if ( itBI != m_BuiltItems.end() )
                {
                  m_BuiltItems.erase( itBI );
                }
                m_Level.SetField( TX, CY, LD12::TILE_SKY_EMPTY );
              }
              CY--;
            }
            m_Level.SetField( TX, TY, LD12::TILE_SKY_EMPTY );
            if ( m_Level.GetField( TX, TY + 1 ) == LD12::TILE_WOODEN_FENCE )
            {
              m_Level.SetField( TX, TY + 1, LD12::TILE_WOODEN_FENCE_TOP );
            }

            m_DraggingBomb = true;
            m_pCurrentObject = m_Level.CreateObject( LD12::GO_WOODEN_FENCE, iX, iY + (int)m_Level.m_LevelDisplayOffset );
            std::map<std::pair<int,int>,LD12::TileTypes>::iterator    itBI( m_BuiltItems.find( std::make_pair( TX, TY ) ) );
            if ( itBI != m_BuiltItems.end() )
            {
              m_BuiltItems.erase( itBI );
            }
            m_Level.m_SoundRequests.insert( LD12::SOUND_GET_ITEM );
          }
          else
          {
            Level::tListObjects::iterator   itB( m_Level.m_Objects.begin() );
            while ( itB != m_Level.m_Objects.end() )
            {
              GameObject& Obj( *itB );

              if ( ( Obj.m_Type == LD12::GO_TIMEBOMB )
              ||   ( Obj.m_Type == LD12::GO_WOODEN_FENCE )
              ||   ( Obj.m_Type == LD12::GO_PLASTIC_FENCE ) )
              {
                if ( ( abs( Obj.m_DispX - iX ) < 12 )
                &&   ( abs( Obj.m_DispY - ( iY + (int)m_Level.m_LevelDisplayOffset ) ) < 12 ) )
                {
                  m_DraggingBomb = true;
                  m_pCurrentObject = &Obj;
                  m_Level.m_SoundRequests.insert( LD12::SOUND_GET_ITEM );
                  // remove from current setup
                  Level::tListObjects::iterator   itCS( m_CurrentSetup.begin() );
                  while ( itCS != m_CurrentSetup.end() )
                  {
                    GameObject& PrevObj( *itCS );

                    if ( PrevObj.m_ObjIndex == Obj.m_ObjIndex )
                    {
                      m_CurrentSetup.erase( itCS );
                      break;
                    }
                    ++itCS;
                  }
                  break;
                }
              }

              ++itB;
            }
          }
        }
      }
      if ( ( iX >= 120 )
      &&   ( iX < 120 + 120 )
      &&   ( iY >= 440 )
      &&   ( iY < 480 ) )
      {
        // Go!
        m_BuildUpMode = false;
        srand( 17 );
        RecalcStructures();
      }
    }
    else
    {
      if ( ( (int)( m_Level.m_CurrentHouseTiles * 100 / m_Level.m_InitialHouseTiles ) <= m_Level.m_PercentageNeeded )
      &&   ( m_DamageDone <= m_Money ) )
      {
        // Success
        if ( ( iX >= 320 - 130 )
        &&   ( iX < 320 - 130 + 120 )
        &&   ( iY >= 60 + 260 )
        &&   ( iY < 60 + 260 + 40 ) )
        {
          m_CurLevel++;
          PrepareLevel( m_CurLevel );
          m_Done        = false;
          m_BuildUpMode = true;
        }
        else if ( ( iX >= 320 + 10 )
        &&        ( iX < 320 + 10 + 120 )
        &&        ( iY >= 60 + 260 )
        &&        ( iY < 60 + 260 + 40 ) )
        {
          RetryLevel();
        }
      }
      else
      {
        if ( ( iX >= 260 )
        &&   ( iX < 260 + 120 )
        &&   ( iY >= 60 + 260 )
        &&   ( iY < 60 + 260 + 40 ) )
        {
          RetryLevel();
        }
      }
    }
  }

  if ( !( iButtons & 1 ) )
  {
    m_ButtonReleased = true;
    if ( m_DisplayNameTime > 0.0f )
    {
      return;
    }
    if ( ( m_BuildUpMode )
    &&   ( m_pCurrentObject ) )
    {
      // stop dragging
      if ( iY >= 440 )
      {
        // dropped onto the bar, just get rid of it
        m_Money += ItemPrice( m_pCurrentObject->m_Type );
        m_Level.RemoveObject( m_pCurrentObject );
      }
      else
      {
        // drop item on game field
        switch ( m_pCurrentObject->m_Type )
        {
          case LD12::GO_TIMEBOMB:
            {
              int TX = iX / 32;
              int TY = ( iY + (int)m_Level.m_LevelDisplayOffset ) / 32;
              if ( ( m_Level.GetField( TX, TY ) == LD12::TILE_SKY_EMPTY )
              ||   ( m_Level.GetField( TX, TY ) == LD12::TILE_EMPTY )
              ||   ( m_Level.GetField( TX, TY ) == LD12::TILE_GRASS ) )
              {
                // cannot build that here
                m_Money += ItemPrice( m_pCurrentObject->m_Type );
                m_Level.RemoveObject( m_pCurrentObject );
                m_Level.m_SoundRequests.insert( LD12::SOUND_CANNOT_PUT_ITEM );
              }
              else
              {
                m_pCurrentObject->m_Hidden = false;
                m_CurrentSetup.push_back( *m_pCurrentObject );
                m_Level.m_SoundRequests.insert( LD12::SOUND_SET_ITEM );
              }
            }
            break;
          case LD12::GO_PLASTIC_FENCE:
            {
              int TX = iX / 32;
              int TY = ( iY + (int)m_Level.m_LevelDisplayOffset ) / 32;
              if ( ( m_Level.GetField( TX, TY ) != LD12::TILE_SKY_EMPTY )
              ||   ( m_Level.GetField( TX, TY + 1 ) == LD12::TILE_SKY_EMPTY ) )
              {
                // cannot build that here
                m_Money += ItemPrice( m_pCurrentObject->m_Type );
                m_Level.RemoveObject( m_pCurrentObject );
                m_Level.m_SoundRequests.insert( LD12::SOUND_CANNOT_PUT_ITEM );
              }
              else
              {
                m_BuiltItems[std::make_pair( TX, TY )] = LD12::TILE_PLASTIC_FENCE;
                m_Level.SetField( TX, TY, LD12::TILE_PLASTIC_FENCE );
                if ( m_Level.GetField( TX, TY + 1 ) == LD12::TILE_WOODEN_FENCE_TOP )
                {
                  m_Level.SetField( TX, TY + 1, LD12::TILE_WOODEN_FENCE );
                }
                m_Level.RemoveObject( m_pCurrentObject );
                m_Level.m_SoundRequests.insert( LD12::SOUND_SET_ITEM );
              }
            }
            break;
          case LD12::GO_WOODEN_FENCE:
            {
              int TX = iX / 32;
              int TY = ( iY + (int)m_Level.m_LevelDisplayOffset ) / 32;
              if ( ( m_Level.GetField( TX, TY ) != LD12::TILE_SKY_EMPTY )
              ||   ( m_Level.GetField( TX, TY + 1 ) == LD12::TILE_SKY_EMPTY ) )
              {
                // cannot build that here
                m_Money += ItemPrice( m_pCurrentObject->m_Type );
                m_Level.RemoveObject( m_pCurrentObject );
                m_Level.m_SoundRequests.insert( LD12::SOUND_CANNOT_PUT_ITEM );
              }
              else
              {
                m_BuiltItems[std::make_pair( TX, TY )] = LD12::TILE_WOODEN_FENCE;
                if ( m_Level.GetField( TX, TY + 1 ) == LD12::TILE_WOODEN_FENCE_TOP )
                {
                  m_Level.SetField( TX, TY + 1, LD12::TILE_WOODEN_FENCE );
                }
                m_Level.SetField( TX, TY, LD12::TILE_WOODEN_FENCE_TOP );
                m_Level.RemoveObject( m_pCurrentObject );
                m_Level.m_SoundRequests.insert( LD12::SOUND_SET_ITEM );
              }
            }
            break;
        }
      }
      m_DraggingBomb    = false;
      m_pCurrentObject  = NULL;
    }
  }

  if ( m_BuildUpMode )
  {
    if ( m_pCurrentObject != NULL )
    {
      m_pCurrentObject->m_DispX = iX;
      m_pCurrentObject->m_DispY = iY + (int)m_Level.m_LevelDisplayOffset;
    }
  }

}



int CGSGame::ItemPrice( LD12::GameObjectType Type )
{

  switch ( Type )
  {
    case LD12::GO_TIMEBOMB:
      return 50;
    case LD12::GO_WOODEN_FENCE:
      return 250;
    case LD12::GO_PLASTIC_FENCE:
      return 20;
  }
  return 0;

}



void CGSGame::OnMouseWheel( int Delta )
{

  if ( m_Level.m_Height <= 15 )
  {
    m_Level.m_LevelDisplayOffset = 0.0f;
    return;
  }
  if ( Delta < 0 )
  {
    if ( m_Level.m_LevelDisplayOffset < m_Level.m_Height * 32.0f - 440 )
    {
      m_Level.m_LevelDisplayOffset += 16.0f;
      if ( m_Level.m_LevelDisplayOffset > m_Level.m_Height * 32.0f - 440 )
      {
        m_Level.m_LevelDisplayOffset = m_Level.m_Height * 32.0f - 440;
      }
    }
  }
  else if ( Delta > 0 )
  {
    if ( m_Level.m_LevelDisplayOffset > 0.0f )
    {
      m_Level.m_LevelDisplayOffset -= 16.0f;
      if ( m_Level.m_LevelDisplayOffset < 0.0f )
      {
        m_Level.m_LevelDisplayOffset = 0.0f;
      }
    }
  }

}