#include ".\gameobject.h"
#include ".\LD12.h"
#include ".\Level.h"



GameObject::GameObject() :
  m_SpeedX( 0.0f ),
  m_SpeedY( 0.0f ),
  m_AccX( 0.0f ),
  m_AccY( 0.0f ),
  m_Angle( 0.0f ),
  m_AngleDelta( 0.0f ),
  m_Texture( LD12::TEX_BOMB ),
  m_Type( LD12::GO_TIMEBOMB ),
  m_LifeTime( 0.0f ),
  m_RemoveMe( false ),
  m_DispX( 0 ),
  m_DispY( 0 ),
  m_FractX( 0.0f ),
  m_FractY( 0.0f ),
  m_Number( 0 ),
  m_ObjIndex( 0 ),
  m_Hidden( false ),
  m_StartDelay( 0.0f ),
  m_Power( 1 ),
  m_Collateral( false ),
  m_ParticleHurtDelay( 0.0f )
{
}



GameObject::~GameObject(void)
{
}



void GameObject::Render( Level& theLevel )
{

  if ( ( m_Type == LD12::GO_SMOKE_LEFT )
  ||   ( m_Type == LD12::GO_SMOKE_RIGHT ) )
  {
  }
  else
  {
    theApp.RenderTextureSectionRotated( m_DispX, m_DispY - (int)theLevel.m_LevelDisplayOffset, theApp.m_TexSec[m_Texture], m_Angle );
  }

}



void GameObject::RenderFX( Level& theLevel )
{

  if ( ( m_Type == LD12::GO_SMOKE_LEFT )
  ||   ( m_Type == LD12::GO_SMOKE_RIGHT ) )
  {
    theApp.RenderTextureSectionRotated( m_DispX, m_DispY - (int)theLevel.m_LevelDisplayOffset, theApp.m_TexSec[m_Texture], m_Angle );
  }

}



void GameObject::Update( float ElapsedTime, Level& theLevel )
{

  if ( m_StartDelay > 0.0f )
  {
    m_StartDelay -= ElapsedTime;
    if ( m_StartDelay <= 0.0f )
    {
      // TODO - start event handling
      m_StartDelay = 0.0f;
    }
    return;
  }
  if ( m_ParticleHurtDelay > 0.0f )
  {
    m_ParticleHurtDelay -= ElapsedTime;
    if ( m_ParticleHurtDelay <= 0.0f )
    {
      m_ParticleHurtDelay = 0.0f;
    }
  }

  switch ( m_Type )
  {
    case LD12::GO_TIMEBOMB:
      // Explode
      theLevel.m_SoundRequests.insert( LD12::SOUND_EXPLOSION );
      theLevel.Explode( this );
      m_Texture = LD12::TEX_EXPLOSION_1;
      m_Type    = LD12::GO_EXPLOSION;
      m_LifeTime  = 0.0f;
      break;
    case LD12::GO_PARTICLE_BETON:
    case LD12::GO_PARTICLE_GLASS:
      m_SpeedX += m_AccX * 0.2f;
      m_SpeedY += m_AccY * 0.2f;
      m_AccX *= 0.8f;
      m_AccY *= 0.8f;
      m_Angle += m_AngleDelta * ElapsedTime;
      m_LifeTime -= ElapsedTime;
      if ( m_LifeTime <= 0.0f )
      {
        m_RemoveMe = true;
      }
      if ( !theLevel.MoveObject( this, m_SpeedX * ElapsedTime, m_SpeedY * ElapsedTime ) )
      {
      }
      break;
    case LD12::GO_EXPLOSION:
      m_LifeTime += ElapsedTime;
      if ( m_LifeTime >= 0.5f )
      {
        m_RemoveMe = true;
      }
      else
      {
        m_Texture = (LD12::TextureTypes)( LD12::TEX_EXPLOSION_1 + (int)( m_LifeTime / ( 0.5f / 3.0f ) ) );
      }
      break;
    case LD12::GO_SMOKE_LEFT:
    case LD12::GO_SMOKE_RIGHT:
      m_LifeTime -= ElapsedTime;
      if ( m_LifeTime <= 0.0f )
      {
        m_RemoveMe = true;
      }
      else
      {
        if ( m_Type == LD12::GO_SMOKE_LEFT )
        {
          if ( m_LifeTime <= 0.35f )
          {
            m_Texture = LD12::TEX_SMOKE_LEFT_3;
          }
          else if ( m_LifeTime <= 0.75f )
          {
            m_Texture = LD12::TEX_SMOKE_LEFT_2;
          }
          theLevel.MoveObject( this, -3.0f * ElapsedTime, -14.0f * ElapsedTime );
        }
        else
        {
          if ( m_LifeTime <= 0.35f )
          {
            m_Texture = LD12::TEX_SMOKE_RIGHT_3;
          }
          else if ( m_LifeTime <= 0.75f )
          {
            m_Texture = LD12::TEX_SMOKE_RIGHT_2;
          }
          theLevel.MoveObject( this, 3.0f * ElapsedTime, -14.0f * ElapsedTime );
        }
      }
      
      break;
  }

}



void GameObject::UpdatePhysicsStep( Level& theLevel )
{

  switch ( m_Type )
  {
    case LD12::GO_PARTICLE_BETON:
    case LD12::GO_PARTICLE_GLASS:
      m_AccY += 2.0f;
      break;
  }

}