#pragma once

#include <list>
#include <vector>

#include "GameObject.h"



class Level
{

  public:

    struct TileInfo
    {
      int     Strength;
      int     InnerStrength;


      TileInfo() :
        Strength( 100 ),
        InnerStrength( 100 )
      {
      }
    };

    typedef std::list<GameObject>   tListObjects;

    tListObjects            m_Objects;

    std::vector<LD12::TileTypes>    m_Field;
    std::vector<TileInfo>   m_Structure;

    int                     m_MoneyAvailable;

    float                   m_PhysicsStep;

    std::string             m_Name;

    int                     m_ObjIndex;

    int                     m_Height;

    float                   m_LevelDisplayOffset;

    int                     m_InitialHouseTiles;
    int                     m_CurrentHouseTiles;
    int                     m_InitialOtherHouseTiles;
    int                     m_CurrentOtherHouseTiles;

    int                     m_PercentageNeeded;

    std::set<LD12::SoundTypes>    m_SoundRequests;


    Level();
    ~Level();


    void                    Render();
    void                    Update( float ElapsedTime );

    bool                    Load( int Nr );
    bool                    Save( int Nr );

    void                    Clear();

    GameObject*             CreateObject( LD12::GameObjectType ObjType, int X = -100, int Y = -100 );

    void                    RemoveObject( GameObject* pObj );

    bool                    MoveObject( GameObject* pObj, float DX, float DY );
    bool                    IsBlocked( GameObject* pObj, int X, int Y, int W, int H, int& BlockedX, int& BlockedY );
    void                    OnObjectBlocked( GameObject* pObj, int TX, int TY );

    bool                    CanAreaContainBomb( int X, int Y, int W, int H );

    void                    SetField( int X, int iY, LD12::TileTypes Tile );
    LD12::TileTypes         GetField( int X, int iY );

    TileInfo&               Structure( int X, int iY );

    void                    Resize( int NewHeight );

    void                    Explode( GameObject* pBomb );
    void                    CollapseLevel();

    void                    DamageStructure( int X, int Y, float Damage );
    void                    DamageInnerStructure( int X, int Y, float Damage );

    bool                    CanObjectBeBlocked( GameObject* pObj );

    void                    ShatterGlass( int X, int Y );
    void                    ShatterBeton( int X, int Y );

    void                    CollateralParticle( GameObject* pObj );

    void                    PlayRequestedSounds();

};
